/* 
 *
 * Based on code from libcdaudio 0.5.0 (Copyright (C)1998 Tony Arcieri)
 *
 * Based on code from Grip 3.0.5 Copyright (c) 1998-2002  Mike Oliphant <oliphant@gtk.org>
 *
 *  Copyright (c) 2003 Gary Baker ozscap@bigpond.com
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */




/* Used with disc_info */
#define CDAUDIO_PLAYING				0
#define CDAUDIO_PAUSED				1
#define CDAUDIO_COMPLETED			2
#define CDAUDIO_NOSTATUS			3

#define MAX_TRACKS				100


/* Used for keeping track of times */
typedef struct _disc_time {
   int mins;
   int secs;
} DiscTime;

/* Track specific information */
typedef struct _track_info {
  DiscTime length;
  DiscTime start_pos;
  int num_frames;
  int start_frame;
  unsigned char flags;
} TrackInfo;

/* Disc information such as current track, amount played, etc */
typedef struct _disc_info {
  int cd_desc;                              /* CD device file desc. */
  char *devname;                            /* CD device file pathname */
  int have_info;                       /* Do we have disc info yet? */
  int disc_present;	            /* Is disc present? */
  int disc_mode;		            /* Current disc mode */
  DiscTime track_time;		            /* Current track time */
  DiscTime disc_time;		            /* Current disc time */
  DiscTime length;		            /* Total disc length */
  int curr_frame;			    /* Current frame */
  int curr_track;		            /* Current track */
  int num_tracks;		            /* Number of tracks on disc */
  TrackInfo track[MAX_TRACKS];	            /* Track specific information */
} DiscInfo;

/* Channle volume structure */
typedef struct _channel_volume { 
  int left;
  int right;
} ChannelVolume;

/* Volume structure */
typedef struct _disc_volume {
   ChannelVolume vol_front;
   ChannelVolume vol_back;
} DiscVolume;


/* Track database structure */

typedef struct _track_data {
  char track_name[256];	              /* Track name */
  char track_artist[256];	      /* Track artist */
  char track_extended[4096];	      /* Extended information */
} TrackData;

/* Disc database structure */

typedef struct _disc_data {
  unsigned int data_id;	              /* CD id */
  char data_title[256];	              /* Disc title */
  char data_artist[256];	      /* Disc artist */
  char data_extended[4096];	      /* Extended information */
  int data_genre;		      /* Discdb genre */
  int data_id3genre;                  /* ID3 genre */
  int data_year;                      /* Disc year */
  char data_playlist[256];            /* Playlist info */
  int data_multi_artist;         /* Is CD multi-artist? */
  TrackData data_track[MAX_TRACKS];   /* Track names */
  int revision;                       /* Database revision */
} DiscData;


