/*
Sonic Rainbow written by Gary Baker South Australia.
Copyright (C) 2003 Gary Baker

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>




#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include <xine.h>
#include <xine/xineutils.h>

GtkWidget* create_single_file_window (char *sd);
void do_error_message(char *text);
void on_vfcombo_entry_changed                (GtkEditable     *editable,
                                        gpointer         user_data);
extern guint cd_timer;
extern char default_video[1000];
extern GtkObject *cadj;
extern GtkWidget *vfcombo_entry;
extern char *fdata;

int dvd_running=0;
int last_speed;

GtkWidget *pf, *vco;

static xine_t *xine;
static xine_stream_t *stream=NULL;
static xine_video_port_t   *vo_port;
static xine_audio_port_t   *ao_port;
static xine_event_queue_t  *event_queue;

x11_visual_t      vis;

static Display             *display;
static int                  screen;
static Window               window[2];
static int                  xpos, ypos, width, height, fullscreen;
static double               pixel_aspect;
double            res_h, res_v;

GtkWindow *toplevel;

int play_cdauto();
void main_vol( GtkAdjustment *get, gpointer         user_data );
void cont_change( GtkAdjustment *get, gpointer         user_data );
void bright_change( GtkAdjustment *get, gpointer         user_data );


/* this will be called by xine, if it wants to know the target size of a frame */
static void dest_size_cb(void *data, int video_width, int video_height, double video_pixel_aspect,
                         int *dest_width, int *dest_height, double *dest_pixel_aspect)  {
  *dest_width        = width;
  *dest_height       = height;
  *dest_pixel_aspect = pixel_aspect;
}

/* this will be called by xine when it's about to draw the frame */
static void frame_output_cb(void *data, int video_width, int video_height,
                            double video_pixel_aspect, int *dest_x, int *dest_y,
                            int *dest_width, int *dest_height,
                            double *dest_pixel_aspect, int *win_x, int *win_y) {
  *dest_x            = 0;
  *dest_y            = 0;
  *win_x             = xpos;
  *win_y             = ypos;
  *dest_width        = width;
  *dest_height       = height;
  *dest_pixel_aspect = pixel_aspect;
}




void
on_vp_play_dvd_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  char              configfile[2048];
  char             *vo_driver = "auto";
  char             *ao_driver = "auto";
  char *t;
  char *ebuff;

  if(dvd_running)return;
  dvd_running=1;

  xine = xine_new();
    if(xine==NULL)
  {
    ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
   dvd_running=0;
  return;
  }

  sprintf(configfile, "%s%s", xine_get_homedir(), "/.xine/config");
  xine_config_load(xine, configfile);
  xine_init(xine);

  t=gtk_entry_get_text((GtkEntry *)vco);
  if(strcmp(t,"320 x 240")==0)
  {
  width   = 320;
  height  = 200;
  }
else
  if(strcmp(t,"640 x 480")==0)
  {
  width   = 640;
  height  = 480;
  }
  else
  if(strcmp(t,"1024 x 768")==0)
  {
  width   = 1024;
  height  = 768;
  }
  xpos    = 0;
  ypos    = 0;


  display = XOpenDisplay (NULL);
  screen = DefaultScreen (display);

    if(strcmp(t,"Full Screen")==0)
  {
  width   = DisplayWidth(display, screen);
  height  = DisplayHeight(display, screen);
  }

  /* some initalization for the X11 Window we will be showing video in */
  XLockDisplay(display);
  fullscreen = 0;
  window[0] = XCreateSimpleWindow(display, XDefaultRootWindow(display),
                                  xpos, ypos, width, height, 1, 0, 0);

 XSelectInput(display, window[0], (ExposureMask | ButtonPressMask | KeyPressMask |
                                    ButtonMotionMask | StructureNotifyMask |
                                    PropertyChangeMask | PointerMotionMask));

  XMapRaised(display, window[0]);

  res_h = (DisplayWidth(display, screen) * 1000 / DisplayWidthMM(display, screen));
  res_v = (DisplayHeight(display, screen) * 1000 / DisplayHeightMM(display, screen));
  XSync(display, False);
  XUnlockDisplay(display);



  vis.display           = display;
  vis.screen            = screen;
  vis.d                 = window[0];
  vis.dest_size_cb      = dest_size_cb;
  vis.frame_output_cb   = frame_output_cb;
  vis.user_data         = NULL;
  pixel_aspect          = res_v / res_h;

  /* opening xine output ports */

  vo_port = xine_open_video_driver(xine, vo_driver, XINE_VISUAL_TYPE_X11, (void *)&vis);
  if(vo_port==NULL)
  {

    ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine Video Driver. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
  dvd_running=0;

  xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);
  XCloseDisplay (display);
  return;
  }

  ao_port = xine_open_audio_driver(xine , ao_driver, NULL);
  if(ao_port==NULL)
  {
      ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine Audio Driver. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
  dvd_running=0;
  xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);
  XCloseDisplay (display);
    return;
  }

    /* open a xine stream connected to these ports */
  stream = xine_stream_new(xine, ao_port, vo_port);
    if(stream==NULL)
  {
    ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine Stream. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
  dvd_running=0;
  xine_close_audio_driver(xine, ao_port);
  xine_close_video_driver(xine, vo_port);
  xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);
  XCloseDisplay (display);
  return;
  }
  /* hook our event handler into the streams events */
  event_queue = xine_event_new_queue(stream);

    /* make the video window visible to xine */

  xine_port_send_gui_data(vo_port, XINE_GUI_SEND_DRAWABLE_CHANGED, (void *) window[0]);

on_vfcombo_entry_changed(NULL,NULL);

  if (!xine_open(stream,"dvd://1") || !xine_play(stream, 0, 0)) {
     ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot open DVD Player or Play DVD");
   do_error_message(ebuff);
   free(ebuff);
    on_vp_stop_clicked(NULL,NULL);
    return;
    }
}


void
on_vp_pause_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
int s;
if(!dvd_running)return;
s=xine_get_param(stream,XINE_PARAM_SPEED);
if(s!=0)
{
last_speed=s;
xine_set_param(stream,XINE_PARAM_SPEED,XINE_SPEED_PAUSE);
}
}


void
on_vp_stop_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  if(!dvd_running)return;
  if(dvd_running==2)
  cd_timer=gtk_timeout_add(250,play_cdauto,NULL);
  dvd_running=0;
  xine_close(stream);
  xine_event_dispose_queue(event_queue);
  xine_dispose(stream);
  xine_close_audio_driver(xine, ao_port);
  xine_close_video_driver(xine, vo_port);
  xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);

  XCloseDisplay (display);

}




void
on_vp_play_vcd_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  char              configfile[2048];
  char             *vo_driver = "auto";
  char             *ao_driver = "auto";
  char *t;
  char *ebuff;

  if(dvd_running)return;
  dvd_running=2;
   gtk_timeout_remove(cd_timer);

  xine = xine_new();
  if(xine==NULL)
  {
    ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
   dvd_running=0;
  cd_timer=gtk_timeout_add(250,play_cdauto,NULL);
  return;
  }


  sprintf(configfile, "%s%s", xine_get_homedir(), "/.xine/config");
  xine_config_load(xine, configfile);
  xine_init(xine);

 t=gtk_entry_get_text((GtkEntry *)vco);
  if(strcmp(t,"320 x 240")==0)
  {
  width   = 320;
  height  = 200;
  }
else
  if(strcmp(t,"640 x 480")==0)
  {
  width   = 640;
  height  = 480;
  }
  else
  if(strcmp(t,"1024 x 768")==0)
  {
  width   = 1024;
  height  = 768;
  }
  xpos    = 0;
  ypos    = 0;




  display = XOpenDisplay (NULL);
  screen = DefaultScreen (display);

      if(strcmp(t,"Full Screen")==0)
  {
  width   = DisplayWidth(display, screen);
  height  = DisplayHeight(display, screen);
  }

  /* some initalization for the X11 Window we will be showing video in */
  XLockDisplay(display);
  fullscreen = 0;
  window[0] = XCreateSimpleWindow(display, XDefaultRootWindow(display),
                                  xpos, ypos, width, height, 1, 0, 0);

 XSelectInput(display, window[0], (ExposureMask | ButtonPressMask | KeyPressMask |
                                    ButtonMotionMask | StructureNotifyMask |
                                    PropertyChangeMask | PointerMotionMask));

  XMapRaised(display, window[0]);

  res_h = (DisplayWidth(display, screen) * 1000 / DisplayWidthMM(display, screen));
  res_v = (DisplayHeight(display, screen) * 1000 / DisplayHeightMM(display, screen));
  XSync(display, False);
  XUnlockDisplay(display);



  vis.display           = display;
  vis.screen            = screen;
  vis.d                 = window[0];
  vis.dest_size_cb      = dest_size_cb;
  vis.frame_output_cb   = frame_output_cb;
  vis.user_data         = NULL;
  pixel_aspect          = res_v / res_h;

  /* opening xine output ports */

  vo_port = xine_open_video_driver(xine, vo_driver, XINE_VISUAL_TYPE_X11, (void *)&vis);

  if(vo_port==NULL)
  {

    ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine Video Driver. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
  dvd_running=0;
  xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);
  XCloseDisplay (display);
  cd_timer=gtk_timeout_add(250,play_cdauto,NULL);
  return;

  }



  ao_port = xine_open_audio_driver(xine , ao_driver, NULL);

  if(ao_port==NULL)
  {
      ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine Audio Driver. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
  dvd_running=0;
/*  xine_close_video_driver(xine, vo_port);*/
    xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);
  XCloseDisplay (display);

  cd_timer=gtk_timeout_add(250,play_cdauto,NULL);

  return;
  }


    /* open a xine stream connected to these ports */
  stream = xine_stream_new(xine, ao_port, vo_port);
  if(stream==NULL)
  {
    ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine Stream. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
  dvd_running=0;
  xine_close_audio_driver(xine, ao_port);
  xine_close_video_driver(xine, vo_port);
  xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);
  XCloseDisplay (display);
  cd_timer=gtk_timeout_add(250,play_cdauto,NULL);
  return;
  }

  /* hook our event handler into the streams events */
  event_queue = xine_event_new_queue(stream);

    /* make the video window visible to xine */

  xine_port_send_gui_data(vo_port, XINE_GUI_SEND_DRAWABLE_CHANGED, (void *) window[0]);


on_vfcombo_entry_changed(NULL,NULL);

  if (!xine_open(stream,"vcd://1") || !xine_play(stream, 0, 0)) {
    char *ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot open CDROM or play VCD");
   do_error_message(ebuff);
   free(ebuff);
    on_vp_stop_clicked(NULL,NULL);
    return;
    }
}


void
on_vp_res_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
int s;
if(!dvd_running)return;
s=xine_get_param(stream,XINE_PARAM_SPEED);
if(s==0)
xine_set_param(stream,XINE_PARAM_SPEED,last_speed);
}


void
on_vp_rewind_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
int s;
if(!dvd_running)return;
s=xine_get_param(stream,XINE_PARAM_SPEED);
if(s==XINE_SPEED_PAUSE)return;

if(s==XINE_SPEED_FAST_4)
xine_set_param(stream,XINE_PARAM_SPEED,XINE_SPEED_FAST_2);
else
if(s==XINE_SPEED_FAST_2)
xine_set_param(stream,XINE_PARAM_SPEED,XINE_SPEED_NORMAL);
else
if(s==XINE_SPEED_NORMAL)
xine_set_param(stream,XINE_PARAM_SPEED,XINE_SPEED_SLOW_2);
else
if(s==XINE_SPEED_SLOW_2)
xine_set_param(stream,XINE_PARAM_SPEED,XINE_SPEED_SLOW_4);
}


void
on_vp_foward_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
int s;
if(!dvd_running)return;
s=xine_get_param(stream,XINE_PARAM_SPEED);
if(s==XINE_SPEED_PAUSE)return;

if(s==XINE_SPEED_SLOW_4)
xine_set_param(stream,XINE_PARAM_SPEED,XINE_SPEED_SLOW_2);
else
if(s==XINE_SPEED_SLOW_2)
xine_set_param(stream,XINE_PARAM_SPEED,XINE_SPEED_NORMAL);
else
if(s==XINE_SPEED_NORMAL)
xine_set_param(stream,XINE_PARAM_SPEED,XINE_SPEED_FAST_2);
else
if(s==XINE_SPEED_FAST_2)
xine_set_param(stream,XINE_PARAM_SPEED,XINE_SPEED_FAST_4);
}


/*
void
on_vp_file_ok_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
*/

void
on_vp_play_file_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
char *t;
char fi[1000];
char              configfile[2048];
char             *vo_driver = "auto";
char             *ao_driver = "auto";
char *e;
char *ebuff;
GtkWidget *a;

 if(dvd_running)return;
  dvd_running=1;

a=single_file_window (default_video);
 gtk_widget_show(a);
gtk_main();
if(fdata==NULL)
{
free(fdata);
fdata=NULL;
gtk_widget_destroy(a);
return;
}

gtk_widget_destroy(a);
t=fdata;



  xine = xine_new();
      if(xine==NULL)
  {
    ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
   dvd_running=0;
   free(fdata);
   fdata=NULL;
  return;
  }
  sprintf(configfile, "%s%s", xine_get_homedir(), "/.xine/config");
  xine_config_load(xine, configfile);
  xine_init(xine);

e=gtk_entry_get_text((GtkEntry *)vco);
  if(strcmp(e,"320 x 240")==0)
  {
  width   = 320;
  height  = 200;
  }
else
  if(strcmp(e,"640 x 480")==0)
  {
  width   = 640;
  height  = 480;
  }
  else
  if(strcmp(e,"1024 x 768")==0)
  {
  width   = 1024;
  height  = 768;
  }
  xpos    = 0;
  ypos    = 0;


  display = XOpenDisplay (NULL);
  screen = DefaultScreen (display);

      if(strcmp(e,"Full Screen")==0)
  {
  width   = DisplayWidth(display, screen);
  height  = DisplayHeight(display, screen);
  }




  /* some initalization for the X11 Window we will be showing video in */
  XLockDisplay(display);
  fullscreen = 0;
  window[0] = XCreateSimpleWindow(display, XDefaultRootWindow(display),
                                  xpos, ypos, width, height, 1, 0, 0);

 XSelectInput(display, window[0], (ExposureMask | ButtonPressMask | KeyPressMask |
                                    ButtonMotionMask | StructureNotifyMask |
                                    PropertyChangeMask | PointerMotionMask));

  XMapRaised(display, window[0]);

  res_h = (DisplayWidth(display, screen) * 1000 / DisplayWidthMM(display, screen));
  res_v = (DisplayHeight(display, screen) * 1000 / DisplayHeightMM(display, screen));
  XSync(display, False);
  XUnlockDisplay(display);



  vis.display           = display;
  vis.screen            = screen;
  vis.d                 = window[0];
  vis.dest_size_cb      = dest_size_cb;
  vis.frame_output_cb   = frame_output_cb;
  vis.user_data         = NULL;
  pixel_aspect          = res_v / res_h;

  /* opening xine output ports */

  vo_port = xine_open_video_driver(xine, vo_driver, XINE_VISUAL_TYPE_X11, (void *)&vis);
  if(vo_port==NULL)
  {

    ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine Video Driver. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
  dvd_running=0;

  xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);
  XCloseDisplay (display);
  free(fdata);
  fdata=NULL;
  return;
  }
  ao_port = xine_open_audio_driver(xine , ao_driver, NULL);
  if(ao_port==NULL)
  {
      ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine Audio Driver. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
  dvd_running=0;
  xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);
  XCloseDisplay (display);
  free(fdata);
   fdata=NULL;
  return;
  }
    /* open a xine stream connected to these ports */
  stream = xine_stream_new(xine, ao_port, vo_port);
     if(stream==NULL)
  {
    ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot Initialise xine Stream. Xine may not be properly installed. ");
   do_error_message(ebuff);
   free(ebuff);
  dvd_running=0;
  xine_close_audio_driver(xine, ao_port);
  xine_close_video_driver(xine, vo_port);
  xine_exit(xine);

  XLockDisplay(display);
  XUnmapWindow(display, window[0]);
  XDestroyWindow(display, window[0]);
  XUnlockDisplay(display);
  XCloseDisplay (display);
  free(fdata);
  fdata=NULL;
  return;
  }
  /* hook our event handler into the streams events */
  event_queue = xine_event_new_queue(stream);

    /* make the video window visible to xine */

  xine_port_send_gui_data(vo_port, XINE_GUI_SEND_DRAWABLE_CHANGED, (void *) window[0]);


on_vfcombo_entry_changed(NULL,NULL);

strcpy(fi,"file://");
strcat(fi,t);

  if (!xine_open(stream,t) || !xine_play(stream, 0, 0)) {
  ebuff=(char *)malloc(5000);
sprintf(ebuff,"Sonic-Rainbow cannot open file %s",t);
   do_error_message(ebuff);
   free(ebuff);
    on_vp_stop_clicked(NULL,NULL);
    /*gtk_widget_destroy(pf);*/
    free(fdata);
    fdata=NULL;
    return;
    }





/*gtk_widget_destroy(pf);*/
fdata=NULL;
free(fdata);
return;
}


/*
void
on_vp_file_cancel_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
gtk_widget_destroy(pf);
return;
}
*/


void
on_vp_combo_realize                    (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_combo_entry1_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{

}


void
on_combo_entry1_realize                (GtkWidget       *widget,
                                        gpointer         user_data)
{
vco=widget;
}


void
on_vfcombo_entry_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
char *t;

if(dvd_running)
{
  t=gtk_entry_get_text((GtkEntry *)vfcombo_entry);
  if(strcmp(t,"1:1")==0)
  xine_set_param(stream,XINE_PARAM_VO_ASPECT_RATIO,XINE_VO_ASPECT_SQUARE);
else
  if(strcmp(t,"4:3")==0)
  xine_set_param(stream,XINE_PARAM_VO_ASPECT_RATIO,XINE_VO_ASPECT_4_3);
  else
  if(strcmp(t,"16:9")==0)
  xine_set_param(stream,XINE_PARAM_VO_ASPECT_RATIO,XINE_VO_ASPECT_ANAMORPHIC);
    else
  if(strcmp(t,"2.11:1")==0)
  xine_set_param(stream,XINE_PARAM_VO_ASPECT_RATIO,XINE_VO_ASPECT_DVB);
}

}

void cont_change( GtkAdjustment *get, gpointer         user_data )
{
if(dvd_running&&stream!=NULL)
xine_set_param(stream,XINE_PARAM_VO_CONTRAST,get->value*1000);
}

void bright_change( GtkAdjustment *get, gpointer         user_data )
{
if(dvd_running&&stream!=NULL)
xine_set_param(stream,XINE_PARAM_VO_BRIGHTNESS,get->value*1000);
}
